import "core-js/modules/es7.symbol.async-iterator";
import "core-js/modules/es6.symbol";
import "core-js/modules/web.dom.iterable";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import { types, clone, detach } from 'mobx-state-tree';
import ProductModelBase from './ProductModelBase';
/**
 * A base model for the shopping cart
 */

var CartModelBase = types.model('CartModelBase', {
  items: types.optional(types.array(ProductModelBase), [])
}).views(function (self) {
  return {
    get quantity() {
      var total = 0;
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = self.items[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var item = _step.value;
          total += item.quantity;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return total;
    }

  };
}).actions(function (self) {
  return {
    /**
     * Adds a product to the cart
     * @param {ProductModelBase} product
     */
    add: function add(product) {
      self.items.push(clone(product));
    },

    /**
     * Removes a product from the cart
     * @param {ProductModelBase} product
     */
    remove: function remove(product) {
      detach(product);
    }
  };
});
export default CartModelBase;
//# sourceMappingURL=CartModelBase.js.map