import "core-js/modules/es7.symbol.async-iterator";
import "core-js/modules/es6.symbol";
import "core-js/modules/web.dom.iterable";

/**
 * @license
 * Copyright © 2017-2019 Moov Corporation.  All rights reserved.
 */
import { types } from 'mobx-state-tree';
export var MenuItemModel = types.model('MenuItemModel', {
  text: types.optional(types.string, ''),
  url: types.maybeNull(types.string),
  state: types.frozen(),
  className: types.maybeNull(types.string),
  image: types.maybeNull(types.string),
  items: types.maybeNull(types.array(types.late(function () {
    return MenuItemModel;
  }))),
  root: types.optional(types.boolean, false),
  server: types.optional(types.boolean, false),
  prefetch: types.maybeNull(types.string),
  expanded: false
}).actions(function (self) {
  return {
    toggle: function toggle() {
      self.expanded = !self.expanded;
    },
    collapse: function collapse() {
      self.expanded = false;
    }
  };
});
export var MenuModel = types.model('MenuModel', {
  open: false,
  levels: types.optional(types.array(MenuItemModel), []),
  level: types.optional(types.number, 0)
}).actions(function (self) {
  return {
    /**
     * Closes the menu
     */
    close: function close() {
      self.open = false;
    },

    /**
     * Toggles the open state of the menu
     */
    toggle: function toggle() {
      self.open = !self.open;
    },

    /**
     * Updates the root node
     * @param {Object} root
     */
    setRoot: function setRoot(root) {
      self.levels[0] = MenuItemModel.create(root);
    },

    /**
     * Selects an item in the menu
     * @param {MenuItemModel} item
     * @param {Object} options
     */
    setSelected: function setSelected(item) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      item = MenuItemModel.create(item.toJSON());
      self.level++;

      if (self.levels.length <= self.level) {
        self.levels.push(item);
      } else {
        self.levels[self.level] = item;
      }

      if (options.expandFirstItem && item.items.every(function (itm) {
        return itm.expanded === false;
      })) {
        item.items[0].expanded = true;
      }
    },

    /**
     * Goes back one level
     */
    goBack: function goBack() {
      self.level = Math.max(0, self.level - 1);
      self.collapseAll();
    },

    /**
     * Closes all expandable sections
     */
    collapseAll: function collapseAll() {
      setTimeout(function () {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = self.levels[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var level = _step.value;
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
              for (var _iterator2 = level.items[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                var item = _step2.value;
                item.collapse();
              }
            } catch (err) {
              _didIteratorError2 = true;
              _iteratorError2 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
                  _iterator2.return();
                }
              } finally {
                if (_didIteratorError2) {
                  throw _iteratorError2;
                }
              }
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }, 200);
    }
  };
});
//# sourceMappingURL=MenuModel.js.map