import _objectSpread from "@babel/runtime/helpers/objectSpread";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import "core-js/modules/es6.string.ends-with";
import "regenerator-runtime/runtime";
import "core-js/modules/es6.regexp.search";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import { fetchLatest, StaleResponseError } from '../fetchLatest';
import { abortPrefetches, resumePrefetches } from './serviceWorker';
var doFetch;
/**
 * Fetch's state as json from the specified url
 * @param {String} url The url to fetch
 * @param {Object} options
 * @param {String} options.cache Set to "force-cache" to cache the response in the service worker.  Omit to skip the service worker cache.
 * @return {Object} A state patch
 */

export function fetch(_x) {
  return _fetch.apply(this, arguments);
}
/**
 * Handles a redirect response.  Will do a client side navigation if the URL has the same hostname as the app, otherwise will
 * reload the page.
 * @param {String} url
 */

function _fetch() {
  _fetch = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee2(url) {
    var _ref,
        _ref$cache,
        cache,
        _location,
        href,
        apiVersion,
        result,
        _args2 = arguments;

    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _ref = _args2.length > 1 && _args2[1] !== undefined ? _args2[1] : {}, _ref$cache = _ref.cache, cache = _ref$cache === void 0 ? 'default' : _ref$cache;
            abortPrefetches();
            doFetch = doFetch || fetchLatest(require('isomorphic-unfetch'));
            _location = location, href = _location.href;
            _context2.prev = 4;
            apiVersion = (window.moov || {}).apiVersion;
            _context2.next = 8;
            return doFetch(url, {
              cache: cache || 'default',
              credentials: 'include',
              headers: {
                'x-react-storefront': 'true',
                // allows back end handlers to quickly identify PWA API requests,
                'x-moov-api-version': apiVersion // needed for the service worker to determine the correct runtime cache name and ensure that we're not getting a cached response from a previous api version

              }
            }).then(function (response) {
              var redirected = response.redirected,
                  url = response.url;

              if (redirected) {
                redirectTo(url);
              } else {
                resumePrefetches();
                return response.json();
              }
            });

          case 8:
            result = _context2.sent;

            if (!(result != null && location.href === href)) {
              _context2.next = 11;
              break;
            }

            return _context2.abrupt("return", _objectSpread({
              loading: false
            }, result));

          case 11:
            _context2.next = 20;
            break;

          case 13:
            _context2.prev = 13;
            _context2.t0 = _context2["catch"](4);

            if (!StaleResponseError.is(_context2.t0)) {
              _context2.next = 19;
              break;
            }

            return _context2.abrupt("return", null);

          case 19:
            throw _context2.t0;

          case 20:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this, [[4, 13]]);
  }));
  return _fetch.apply(this, arguments);
}

function redirectTo(url) {
  if (url) {
    var parsed = new URL(url);
    var history = window.moov.history;

    if (parsed.hostname === window.location.hostname) {
      history.push(parsed.pathname + parsed.search);
    } else {
      window.location.assign(url);
    }
  } else {
    throw new Error('Received a redirect without a location header.');
  }
}
/**
 * Creates a handler that fetches data from the server.
 *
 * The `handlerPath` should point to a module that exports a function that takes params, request, and response,
 * and returns an object that should be applied to the app state.  For example:
 *
 *  // routes.js
 *  router.get('/p/:id'
 *    fromServer('./product/product-handler')
 *  )
 *
 *  // product/product-handler.js
 *  export default function productHandler(params, request, response) {
 *    return fetchFromUpstreamApi(`/products/${params.id}`)
 *      .then(res => res.json())
 *      .then(productData => ({ // the shape of this object should match your AppModel
 *        page: 'Product',
 *        product: productData
 *      }))
 *  }
 *
 * When the request path ends in ".json", the json response will be returned verbatim.  In all other cases, server-side rendered HTML
 * will be returned.
 *
 * You can also send a verbatim string response using `response.send(body)`.  For example:
 *
 *  // routes.js
 *  router.get('/my-api'
 *    fromServer('./my-api-handler')
 *  )
 *
 *  // my-api-handler.js
 *  export default function myApiHandler(params, request, response) {
 *    response
 *      .set('content-type', response.JSON)
 *      .send(JSON.stringify({ foo: 'bar' }))
 *  }
 *
 * When `response.send()` is called in a handler, react-storefront will never perform server-side rendering.
 *
 * @param {String} handlerPath The path to the module that exports a handler function that returns state to apply
 * @param {Function} getURL An optional function that returns the back end url to call when fetching.  You only need
 *   to specify this if you want to override the default URL.
 * @return {Function}
 */


export default function fromServer(handlerPath, getURL) {
  if (handlerPath == null) {
    throw new Error('You must provide a path to a handler in fromServer().  Please check your routes.');
  }

  return {
    type: 'fromServer',
    runOn: {
      server: true,
      client: true // fromServer handlers run on the client too - we make an ajax request to get the state from the server

    },
    fn: function () {
      var _fn = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee(params, request, response) {
        var url;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!(typeof handlerPath === 'string')) {
                  _context.next = 6;
                  break;
                }

                url = "".concat(location.pathname, ".json").concat(location.search);

                if (getURL) {
                  url = getURL(url);
                } // handler path has not been transpiled, fetch the data from the server and return the result.


                return _context.abrupt("return", fetch(url, {
                  cache: response.clientCache
                }));

              case 6:
                // indicate handler path and asset class in a response header so we can track it in logs
                response.set('x-rsf-handler', handlerPath.path);
                response.set('x-rsf-response-type', request.path.endsWith('.json') ? 'json' : 'ssr'); // handler path has been transpiled to a function

                return _context.abrupt("return", handlerPath(params, request, response));

              case 9:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function fn(_x2, _x3, _x4) {
        return _fn.apply(this, arguments);
      }

      return fn;
    }()
  };
}
//# sourceMappingURL=fromServer.js.map